"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Channel = void 0;
const Resource_1 = require("../types/Resource");
const Util_1 = require("../util/Util");
const Resource_2 = require("./Resource");
class Channel extends Resource_2.Resource {
    constructor(yt, data) {
        super(yt, data, 'channel');
        switch (data.kind) {
            case Resource_1.ResourceKind.channel:
                this.id = data.id;
                break;
            case Resource_1.ResourceKind.playlist:
            case Resource_1.ResourceKind.playlistItem:
            case Resource_1.ResourceKind.video:
                if (data.snippet) {
                    this.id = data.snippet.channelId;
                }
                else
                    throw new Error('Attempted to make a channel out of a resource with no channel data.');
                break;
            case Resource_1.ResourceKind.searchResult:
                if (data.id.kind === Resource_1.ResourceKind.channel) {
                    this.id = data.id.channelId;
                    break;
                }
                else if (data.snippet) {
                    this.id = data.snippet.channelId;
                    break;
                }
                else
                    throw new Error('Attempted to make a channel out of a search result with no channel data.');
            default:
                throw new Error(`Unknown channel kind: ${data.kind}.`);
        }
        if (this.yt.options.fetchAll)
            this.fetch(data);
        else
            this.patch(data);
    }
    static extractId(url) {
        const parsedUrl = Util_1.parseURL(url);
        if (parsedUrl === null || parsedUrl === void 0 ? void 0 : parsedUrl.channel)
            return parsedUrl.channel;
        return null;
    }
    async fetch(options) {
        const channel = await this.yt.getChannelById(this.id, options);
        return this.patch(channel);
    }
    patch(data) {
        switch (data.kind) {
            case Resource_1.ResourceKind.playlist:
            case Resource_1.ResourceKind.playlistItem:
            case Resource_1.ResourceKind.video:
            case Resource_1.ResourceKind.searchResult:
                if (data.snippet) {
                    this.title = data.snippet.channelTitle;
                }
                else
                    throw new Error('Attempted to make a channel out of a resource with no channel data.');
                break;
            default:
                if (data.snippet) {
                    this.title = data.snippet.title;
                    this.description = data.snippet.description;
                    this.publishedAt = new Date(data.snippet.publishedAt);
                    this.thumbnails = data.snippet.thumbnails;
                }
                if (data.statistics) {
                    this.commentCount = data.statistics.commentCount;
                    this.hiddenSubscriberCount = data.statistics.hiddenSubscriberCount;
                    this.subscriberCount = data.statistics.subscriberCount;
                    this.videoCount = data.statistics.videoCount;
                    this.viewCount = data.statistics.viewCount;
                }
                if (data.contentDetails) {
                    this.relatedPlaylists = data.contentDetails.relatedPlaylists;
                }
        }
        return this;
    }
}
exports.Channel = Channel;
