"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Playlist = void 0;
const Resource_1 = require("../types/Resource");
const Util_1 = require("../util/Util");
const Channel_1 = require("./Channel");
const Resource_2 = require("./Resource");
class Playlist extends Resource_2.Resource {
    constructor(yt, data) {
        super(yt, data, 'playlist');
        switch (data.kind) {
            case Resource_1.ResourceKind.playlist:
                this.id = data.id;
                break;
            case Resource_1.ResourceKind.playlistItem:
                if (data.snippet)
                    this.id = data.snippet.playlistId;
                else
                    throw new Error('Attempted to make a playlist out of a resource with no playlist data.');
                break;
            case Resource_1.ResourceKind.searchResult:
                if (data.id.kind === Resource_1.ResourceKind.playlist)
                    this.id = data.id.playlistId;
                else
                    throw new Error('Attempted to make a playlist out of a search result with no playlist data.');
                break;
            default:
                throw new Error(`Unknown playlist kind: ${data.kind}.`);
        }
        if (this.yt.options.fetchAll)
            this.fetch(data);
        else
            this.patch(data);
    }
    static extractId(url) {
        const parsedUrl = Util_1.parseURL(url);
        if (parsedUrl === null || parsedUrl === void 0 ? void 0 : parsedUrl.playlist)
            return parsedUrl.playlist;
        return null;
    }
    async fetch(options) {
        const playlist = await this.yt.getPlaylistById(this.id, options);
        return this.patch(playlist);
    }
    patch(data) {
        this.channel = new Channel_1.Channel(this.yt, data);
        this.url = `https://www.youtube.com/playlist?list=${this.id}`;
        if (data.snippet) {
            this.title = data.snippet.title;
            this.description = data.snippet.description;
            this.publishedAt = new Date(data.snippet.publishedAt);
            this.publishedTimestamp = data.snippet.publishedAt;
            this.thumbnails = data.snippet.thumbnails;
            this.channelTitle = data.snippet.channelTitle;
            this.defaultLanguage = data.snippet.defaultLanguage;
        }
        if (data.status) {
            this.itemCount = data.status.itemCount;
        }
        if (data.contentDetails) {
            this.itemCount = data.contentDetails.itemCount;
        }
        if (data.player) {
            this.embed = data.player.embedHtml;
        }
        return this;
    }
}
exports.Playlist = Playlist;
