"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Video = void 0;
const Resource_1 = require("../types/Resource");
const Util_1 = require("../util/Util");
const Channel_1 = require("./Channel");
const Resource_2 = require("./Resource");
const iso8601_duration_1 = require("iso8601-duration");
class Video extends Resource_2.Resource {
    constructor(yt, data) {
        super(yt, data, 'video');
        switch (data.kind) {
            case Resource_1.ResourceKind.video:
                this.id = data.id;
                break;
            case Resource_1.ResourceKind.playlistItem:
                if (data.snippet) {
                    if (data.snippet.resourceId.kind === Resource_1.ResourceKind.video)
                        this.id = data.snippet.resourceId.videoId;
                    else
                        throw new Error('Attempted to make a video out of a non-video playlist item.');
                }
                else
                    throw new Error('Attempted to make a video out of a playlist item with no video data.');
                break;
            case Resource_1.ResourceKind.searchResult:
                if (data.id.kind === Resource_1.ResourceKind.video)
                    this.id = data.id.videoId;
                else
                    throw new Error('Attempted to make a video out of a non-video search result.');
                break;
            default:
                throw new Error(`Unknown video kind: ${data.kind}.`);
        }
        this.patch(data);
    }
    static extractId(url) {
        const parsedUrl = Util_1.parseURL(url);
        if (parsedUrl === null || parsedUrl === void 0 ? void 0 : parsedUrl.video)
            return parsedUrl.video;
        return null;
    }
    async fetch(options = {}) {
        const video = await this.yt.getVideoById(this.id, options);
        return this.patch(video);
    }
    async fetchRelated(options = {}) {
        this.relatedVideos = await this.yt.searchVideos('', Object.assign(options, { relatedToVideoId: this.id }));
        return this.relatedVideos;
    }
    patch(data) {
        this.channel = new Channel_1.Channel(this.yt, data);
        this.url = `https://www.youtube.com/watch?v=${this.id}`;
        if (data.snippet) {
            this.title = data.snippet.title;
            this.description = data.snippet.description;
            this.publishedAt = new Date(data.snippet.publishedAt);
            this.publishedTimestamp = data.snippet.publishedAt;
            this.thumbnails = data.snippet.thumbnails;
        }
        if (data.contentDetails) {
            this.duration = data.contentDetails.duration ? iso8601_duration_1.parse(data.contentDetails.duration) : null;
            this.durationSeconds = data.contentDetails.duration ? iso8601_duration_1.toSeconds(data.contentDetails.duration) : null;
        }
        return this;
    }
}
exports.Video = Video;
